@@ -1,7 +1,7 @@
 // This defines the API provided by this library.  It must not be explicitly
 // included by the library itself.
 //
-// Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -25,11 +25,6 @@
 // http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 // or contact the sales department at sales@riverbankcomputing.com.
 // 
-// This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-// INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-// granted herein.
-// 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
@@ -53,12 +48,22 @@ PyObject *qpycore_pyqtslot(PyObject *args, PyObject *kwds);
 PyObject *qpycore_pyqtsignature(PyObject *args, PyObject *kwds);
 
 
+// Support for pyqtConfigure().
+int qpycore_pyqtconfigure(PyObject *self, QObject *qobj, PyObject *kwds);
+
+
 // Support for converting between PyObject and QString.
 PyObject *qpycore_PyObject_FromQString(const QString &qstr);
 QString qpycore_PyObject_AsQString(PyObject *obj);
 const char *qpycore_encode(PyObject **s, QCoreApplication::Encoding encoding);
 
 
+// Support for converting between PyObject and QStringList.
+PyObject *qpycore_PyObject_FromQStringList(const QStringList &qstrlst);
+QStringList qpycore_PySequence_AsQStringList(PyObject *obj);
+int qpycore_PySequence_Check_QStringList(PyObject *obj);
+
+
 // Support for converting between PyObject and QVariant.
 PyObject *qpycore_PyObject_FromQVariant(const QVariant &qvar);
 QVariant qpycore_PyObject_AsQVariant(PyObject *obj, int *is_err);
@@ -78,10 +83,6 @@ PyObject *qpycore_ReturnValue(PyObject *gra);
 PyObject *qpycore_qobject_staticmetaobject(PyObject *type_obj);
 
 
-// Support for QObject.sender().
-QObject *qpycore_qobject_sender();
-
-
 // Support for emitting signals.
 bool qpycore_qobject_emit(QObject *qtx, const char *sig, PyObject *sigargs);
 
@@ -90,6 +91,10 @@ bool qpycore_qobject_emit(QObject *qtx, const char *sig, PyObject *sigargs);
 void qpycore_qmetaobject_connectslotsbyname(QObject *qobj,
         PyObject *qobj_wrapper);
 
+// Support for signals.
+bool qpycore_pyqtsignal_get_parts(PyObject *sig_obj, QObject **qtx,
+        const char **sig);
+
 
 // Utilities.
 #if PY_MAJOR_VERSION >= 3
