@@ -1,6 +1,6 @@
 // This is the interface of the Chimera and related classes.
 //
-// Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -24,11 +24,6 @@
 // http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 // or contact the sales department at sales@riverbankcomputing.com.
 // 
-// This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-// INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-// granted herein.
-// 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
@@ -41,6 +36,7 @@
 
 #include <QByteArray>
 #include <QHash>
+#include <QMetaProperty>
 #include <QVariant>
 
 #include "qpycore_sip.h"
@@ -57,6 +53,9 @@ public:
     // Parses a Python object as a type.  Return 0 if there was an error.
     static const Chimera *parse(PyObject *obj);
 
+    // Parses a Qt meta-property as a type.  Return 0 if there was an error.
+    static const Chimera *parse(const QMetaProperty &mprop);
+
     class Signature
     {
     public:
@@ -73,6 +72,13 @@ public:
         // but excluding a return type).
         QByteArray signature;
 
+        // The signature as declared by the user for use in exceptions.
+        QByteArray py_signature;
+
+        // The optional docstring which will start with '\1' if it is auto
+        // generated and so can be used in exceptions.
+        const char *docstring;
+
         // Return the parsed signature wrapped in a Python object.  Ownership
         // of the signature is passed to the Python object.  Return 0 if there
         // was an error.
@@ -100,7 +106,7 @@ public:
         bool _cached;
 
         Signature(const QByteArray &sig, bool cached)
-            : result(0), signature(sig), _cached(cached) {}
+            : result(0), signature(sig), py_signature(sig), docstring(0), _cached(cached) {}
 
         Signature(const Signature &);
         Signature &operator=(const Signature &);
@@ -147,11 +153,14 @@ public:
         // The state of any temporary stored data.
         int _tmp_state;
 
+        // Set if the value is valid.
+        bool _valid;
+
         Storage(const Chimera *ct);
         Storage(const Chimera *ct, PyObject *py);
 
         // Return true if the value is valid.
-        bool isValid() const {return _value_storage.isValid();}
+        bool isValid() const {return _valid;}
 
         // Return true is the value is a pointer type.
         bool isPointerType() const;
@@ -171,9 +180,9 @@ public:
     // there was an error.
     bool fromPyObject(PyObject *py, void *cpp) const;
 
-    // Convert a Python object to a QVariant (which will be invalid if there
-    // was an error).
-    QVariant fromPyObject(PyObject *py) const;
+    // Convert a Python object to a QVariant.  Return false if there was an
+    // error.
+    bool fromPyObject(PyObject *py, QVariant *var, bool strict = true) const;
 
     // Convert a Python object to a QVariant based on the type of the object.
     static QVariant fromAnyPyObject(PyObject *py, int *is_err);
@@ -191,6 +200,10 @@ public:
     // isn't known to Qt's meta-type system.
     int metatype() const {return _metatype;}
 
+    // Returns a borrowed reference to the Python type object that was used to
+    // define the type (if any).
+    PyObject *py_type() const {return _py_type;}
+
     // Returns the C++ name of the type.
     const QByteArray &name() const {return _name;}
 
@@ -211,6 +224,10 @@ private:
     // but hasn't been wrapped.
     const sipTypeDef *_type;
 
+    // The Python type object if a Python type (rather than a C++ type string)
+    // was given.
+    PyObject *_py_type;
+
     // The Qt meta-type.  This will always be valid but may refer to the
     // PyObject wrapper if the underlying type isn't known to Qt.
     int _metatype;
@@ -230,14 +247,11 @@ private:
     // The cache of previously parsed argument type lists.
     static QHash<QByteArray, QList<const Chimera *> > _previously_parsed;
 
-    typedef void (*AssignFunc)(void *, const void *);
-
     Chimera();
 
     bool parse_cpp_type(const QByteArray &type);
-    bool parse_py_type(PyTypeObject *type_obj,
-            bool use_qvariant_containers=false);
-    AssignFunc get_assign_helper() const;
+    bool parse_py_type(PyTypeObject *type_obj);
+    sipAssignFunc get_assign_helper() const;
     void set_flag();
     bool to_QVariantList(PyObject *py, QVariantList &cpp) const;
     bool to_QVariantMap(PyObject *py, QVariantMap &cpp) const;
